#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import yaml

from django.contrib.staticfiles.templatetags.staticfiles import static
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs


from gbpui import client
from gbpui import column_filters as gfilters

import tables as ns_tables


class ServiceChainSpecTab(tabs.TableTab):
    name = _("Service Chain Specs")
    table_classes = (ns_tables.ServiceChainSpecTable,)
    slug = "service_chain_specs"
    template_name = "horizon/common/_detail_table.html"

    def get_service_chain_spec_table_data(self):
        specs = []
        try:
            specs = client.servicechainspec_list(self.request,
                tenant_id=self.request.user.tenant_id)
            specs = [gfilters.update_sc_spec_attributes(
                self.request, item) for item in specs]
        except Exception:
            pass
        return specs


class ServiceChainNodeTab(tabs.TableTab):
    name = _("Service Chain Nodes")
    table_classes = (ns_tables.ServiceChainNodeTable,)
    slug = "service_chain_node"
    template_name = "horizon/common/_detail_table.html"

    def get_service_chain_node_table_data(self):
        nodes = []
        try:
            nodes = client.servicechainnode_list(self.request,
                tenant_id=self.request.user.tenant_id)
            nodes = [gfilters.update_sc_node_attributes(
                self.request, item) for item in nodes]
        except Exception:
            pass
        return nodes


class ServiceChainInstanceTab(tabs.TableTab):
    name = _("Service Chain Instances")
    table_classes = (ns_tables.ServiceChainInstanceTable,)
    slug = "service_chain_instance"
    template_name = "horizon/common/_detail_table.html"

    def get_service_chain_instance_table_data(self):
        instances = []
        try:
            instances = client.servicechaininstance_list(self.request,
                tenant_id=self.request.user.tenant_id)
            instances = [gfilters.update_sc_instance_attributes(
                self.request, item) for item in instances]
        except Exception:
            pass
        return instances


class ServiceProfileTab(tabs.TableTab):
    name = _("Service Profiles")
    table_classes = (ns_tables.ServiceProfileTable,)
    slug = "service_profile"
    template_name = "horizon/common/_detail_table.html"

    def get_service_profile_table_data(self):
        service_profiles = []
        try:
            service_profiles = client.serviceprofile_list(self.request)
        except Exception:
            pass
        return service_profiles


class ServiceChainTabs(tabs.TabGroup):
    slug = "service_chain_spec_tabs"
    tabs = (ServiceChainSpecTab, ServiceChainNodeTab, ServiceProfileTab,
            ServiceChainInstanceTab)
    sticky = True


class ServiceProfileDetailsTab(tabs.Tab):
    name = _("Service Profile Details")
    slug = "serviceprofile_details"
    template_name = "project/network_services/_serviceprofile_details.html"
    failure_url = reverse_lazy('horizon:project:network_services:index')

    def get_context_data(self, request):
        service_profile_id = self.tab_group.kwargs['sp_id']
        try:
            service_profile = client.get_service_profile(
                request, service_profile_id)
        except Exception:
            exceptions.handle(request, _(
                'Unable to retrieve service profile details.'),
                redirect=self.failure_url)
        return {'service_profile': service_profile}


class ServiceProfileDetailsTabGroup(tabs.TabGroup):
    slug = 'scnode_details'
    tabs = (ServiceProfileDetailsTab,)


class ServiceChainNodeDetailsTab(tabs.Tab):
    name = _("Service Chain Node Details")
    slug = "scnode_details"
    template_name = "project/network_services/_scnode_details.html"
    failure_url = reverse_lazy('horizon:project:network_services:index')

    def get_context_data(self, request):
        scnode_id = self.tab_group.kwargs['scnode_id']
        try:
            scnode = client.get_servicechain_node(request, scnode_id)
            config = scnode.config
            config = config.strip()
            if config.startswith('{'):
                config = yaml.load(config)
                if 'heat_template_version' in config:
                    scnode.config = yaml.dump(
                        config, default_flow_style=False, indent=4)
                else:
                    scnode.config = json.dumps(
                        config, sort_keys=False, indent=4)
            else:
                config = yaml.load(config)
                scnode.config = yaml.dump(
                    config, default_flow_style=False, indent=4)
            scnode.tree = self.prepare_config_as_tree(config)
        except Exception:
            exceptions.handle(request, _(
                'Unable to retrieve service node details.'),
                redirect=self.failure_url)
        return {'scnode': scnode}

    def prepare_config_as_tree(self, config):
        tree = []
        for key, value in config.iteritems():
            node = {}
            if isinstance(value, dict):
                node = self.prepare_root_node(value)
                node["text"] = key
            else:
                node["text"] = key + " : " + str(value)
                node["icon"] = static("dashboard/img/text.png")

            tree.append(node)
        return json.dumps(tree)

    def prepare_root_node(self, obj):
        node = {}
        if obj:
            children = self.prepare_children(obj)
            node["children"] = children
            state = {}
            state["opened"] = True
            node["state"] = state
        return node

    def prepare_children(self, obj):
        children = []
        for key, value in obj.iteritems():
            node = {}
            child = self.json2array(value)
            node["text"] = key
            node["children"] = child
            children.append(node)

        return children

    def json2array(self, obj):
        arr = []
        for key, value in obj.iteritems():
            node = {}
            if isinstance(value, dict):
                children = self.json2array(value)
                node["text"] = key
                node["children"] = children
            elif isinstance(value, list):
                items = []
                for item in value:
                    items.append(json.dumps(item))
                node["children"] = items
                node["text"] = key
            else:
                node["text"] = key + " : " + str(value)
                node["icon"] = static("dashboard/img/text.png")
            arr.append(node)
        return arr


class SCNodeDetailsTabGroup(tabs.TabGroup):
    slug = 'scnode_details'
    tabs = (ServiceChainNodeDetailsTab,)


class ServiceChainSpecDetailsTab(tabs.Tab):
    name = _("Service Chain Spec Details")
    slug = "scspec_details"
    template_name = "project/network_services/_scspec_details.html"
    failure_url = reverse_lazy('horizon:project:network_services:index')

    def get_context_data(self, request):
        scspec_id = self.tab_group.kwargs['scspec_id']
        try:
            scspec = client.get_servicechain_spec(request, scspec_id)
            nodes = []
            gn = lambda x, y: client.get_servicechain_node(x, y)
            for node in scspec.nodes:
                n = gn(self.request, node)
                config = n.config
                config = config.strip()
                if config.startswith('{'):
                    config = yaml.load(config)
                    if 'heat_template_version' in config:
                        config = yaml.dump(
                            config, default_flow_style=False, indent=4)
                    else:
                        config = json.dumps(config, sort_keys=False, indent=4)
                else:
                    config = yaml.load(config)
                    config = yaml.dump(
                        config, default_flow_style=False, indent=4)
                setattr(n, 'config', config)
                nodes.append(n)
            setattr(scspec, 'nodes', nodes)
        except Exception:
            exceptions.handle(request, _(
                'Unable to retrieve service chain spec details.'),
                redirect=self.failure_url)
        return {'scspec': scspec}


class SCSpecDetailsTabGroup(tabs.TabGroup):
    slug = 'scspec_details'
    tabs = (ServiceChainSpecDetailsTab,)


class ServiceChainInstanceDetailsTab(tabs.Tab):
    name = _("Service Chain Instance Details")
    slug = "scinstance_details"
    template_name = "project/network_services/_scinstance_details.html"
    failure_url = reverse_lazy('horizon:project:network_services:index')

    def get_context_data(self, request):
        scinstance_id = self.tab_group.kwargs['scinstance_id']
        try:
            scinstance = client.get_servicechain_instance(
                request, scinstance_id)
        except Exception:
            exceptions.handle(request, _(
                'Unable to retrieve service instance details.'),
                redirect=self.failure_url)
        return {'scinstance': scinstance}


class SCInstanceDetailsTabGroup(tabs.TabGroup):
    slug = 'scinstance_details'
    tabs = (ServiceChainInstanceDetailsTab,)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import re

from django.core.urlresolvers import reverse_lazy
from django.http import HttpResponse  # noqa
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import tabs
from horizon.utils import memoized
from horizon import workflows

from gbpui import client

import forms as policy_target_forms
import tabs as policy_target_tabs
import workflows as policy_target_workflows

from openstack_dashboard import api

from netaddr import IPAddress
from netaddr import IPNetwork

PTGTabs = policy_target_tabs.PTGTabs
PTGDetailsTabs = policy_target_tabs.PTGDetailsTabs

AddPTG = policy_target_workflows.AddPTG
AddExternalPTG = policy_target_workflows.AddExternalPTG


class IndexView(tabs.TabView):
    tab_group_class = (PTGTabs)
    template_name = 'project/policytargets/details_tabs.html'

    def post(self, request, *args, **kwargs):
        obj_ids = request.POST.getlist('object_ids')
        action = request.POST['action']
        obj_type = re.search('delete([a-z]+)', action).group(1)
        if not obj_ids:
            obj_ids.append(re.search('([0-9a-z-]+)$', action).group(1))
        if obj_type == 'policytarget':
            for obj_id in obj_ids:
                try:
                    client.policy_target_delete(request, obj_id)
                    messages.success(request,
                                 _('Deleted Group %s') % obj_id)
                except Exception as e:
                    exceptions.handle(request,
                                  _('Unable to delete Group. %s') % e)
        if obj_type == 'externalpolicytarget':
            for obj_id in obj_ids:
                try:
                    client.ext_policy_target_delete(request, obj_id)
                    messages.success(request,
                                 _('Deleted External Group %s') % obj_id)
                except Exception as e:
                    exceptions.handle(request,
                                  _('Unable to delete External Group. %s') % e)
        return self.get(request, *args, **kwargs)


class AddPTGView(workflows.WorkflowView):
    workflow_class = AddPTG
    template_name = "project/policytargets/addpolicy_target.html"


class AddExternalPTGView(workflows.WorkflowView):
    workflow_class = AddExternalPTG


class PTGDetailsView(tabs.TabbedTableView):
    tab_group_class = (policy_target_tabs.PTGMemberTabs)
    template_name = 'project/policytargets/group_details.html'

    def get_context_data(self, **kwargs):
        context = super(PTGDetailsView, self).get_context_data(**kwargs)
        try:
            policy_target = client.policy_target_get(
                self.request, context['policy_target_id'])
            context['policy_target'] = policy_target
        except Exception:
            pass
        return context


class ExternalPTGDetailsView(tabs.TabbedTableView):
    tab_group_class = (policy_target_tabs.ExternalPTGMemberTabs)
    template_name = 'project/policytargets/group_details.html'

    def get_context_data(self, **kwargs):
        context = super(ExternalPTGDetailsView, self).get_context_data(
            **kwargs)
        try:
            ext_policy_target = client.ext_policy_target_get(
                self.request, context['ext_policy_target_id'])
            context['policy_target'] = ext_policy_target
        except Exception:
            pass
        return context


class LaunchVMView(workflows.WorkflowView):
    workflow_class = policy_target_workflows.LaunchInstance
    template_name = "project/policytargets/add_vm.html"

    def get_initial(self):
        initial = super(LaunchVMView, self).get_initial()
        initial['project_id'] = self.request.user.tenant_id
        initial['user_id'] = self.request.user.id
        return initial


class UpdatePTGView(forms.ModalFormView):
    form_class = policy_target_forms.UpdatePolicyTargetForm
    template_name = "project/policytargets/update_policy_target.html"
    context_object_name = 'policy_target'
    success_url = reverse_lazy("horizon:project:policytargets:index")

    def get_context_data(self, **kwargs):
        context = super(UpdatePTGView, self).get_context_data(**kwargs)
        context["policy_target_id"] = self.kwargs['policy_target_id']
        obj = self._get_object()
        if obj:
            context['name'] = obj.name
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        policy_target_id = self.kwargs['policy_target_id']
        try:
            policy_target = client.policy_target_get(
                self.request, policy_target_id)
            policy_target.set_id_as_name_if_empty()
            return policy_target
        except Exception:
            redirect = self.success_url
            msg = _('Unable to retrieve policy_target details.')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        return self.kwargs


class UpdateExternalPTGView(forms.ModalFormView):
    form_class = policy_target_forms.UpdateExternalPolicyTargetForm
    template_name = "project/policytargets/update_external_policy_target.html"
    context_object_name = 'external_policy_target'
    success_url = reverse_lazy("horizon:project:policytargets:index")

    def get_context_data(self, **kwargs):
        context = super(UpdateExternalPTGView, self).get_context_data(**kwargs)
        context["ext_policy_target_id"] = self.kwargs['ext_policy_target_id']
        obj = self._get_object()
        if obj:
            context['name'] = obj.name
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        ext_policy_target_id = self.kwargs['ext_policy_target_id']
        try:
            ext_policy_target = client.ext_policy_target_get(
                self.request, ext_policy_target_id)
            ext_policy_target.set_id_as_name_if_empty()
            return ext_policy_target
        except Exception:
            redirect = self.success_url
            msg = _('Unable to retrieve policy_target details.')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        return self.kwargs


class ExtAddProvidedPRSView(forms.ModalFormView):
    form_class = policy_target_forms.ExtAddProvidedPRSForm
    template_name = "project/policytargets/ext_add_provided_prs.html"

    def get_context_data(self, **kwargs):
        context = super(ExtAddProvidedPRSView, self).get_context_data(**kwargs)
        context["ext_policy_target_id"] = self.kwargs['ext_policy_target_id']
        return context

    def get_initial(self):
        return self.kwargs


class ExtRemoveProvidedPRSView(forms.ModalFormView):
    form_class = policy_target_forms.ExtRemoveProvidedPRSForm
    template_name = \
        "project/policytargets/ext_remove_provided_prs.html"

    def get_context_data(self, **kwargs):
        context = super(ExtRemoveProvidedPRSView, self).get_context_data(
            **kwargs)
        context["ext_policy_target_id"] = self.kwargs['ext_policy_target_id']
        return context

    def get_initial(self):
        return self.kwargs


class AddProvidedPRSView(forms.ModalFormView):
    form_class = policy_target_forms.AddProvidedPRSForm
    template_name = "project/policytargets/add_provided_prs.html"

    def get_context_data(self, **kwargs):
        context = super(AddProvidedPRSView, self).get_context_data(**kwargs)
        context["policy_target_id"] = self.kwargs['policy_target_id']
        return context

    def get_initial(self):
        return self.kwargs


class RemoveProvidedPRSView(forms.ModalFormView):
    form_class = policy_target_forms.RemoveProvidedPRSForm
    template_name = "project/policytargets/remove_provided_prs.html"

    def get_context_data(self, **kwargs):
        context = super(RemoveProvidedPRSView, self).get_context_data(**kwargs)
        context["policy_target_id"] = self.kwargs['policy_target_id']
        return context

    def get_initial(self):
        return self.kwargs


class ExtAddConsumedPRSView(forms.ModalFormView):
    form_class = policy_target_forms.ExtAddConsumedPRSForm
    template_name = "project/policytargets/ext_add_consumed_prs.html"

    def get_context_data(self, **kwargs):
        context = super(ExtAddConsumedPRSView, self).get_context_data(**kwargs)
        context["ext_policy_target_id"] = self.kwargs['ext_policy_target_id']
        return context

    def get_initial(self):
        return self.kwargs


class ExtRemoveConsumedPRSView(forms.ModalFormView):
    form_class = policy_target_forms.ExtRemoveConsumedPRSForm
    template_name = \
        "project/policytargets/ext_remove_consumed_prs.html"

    def get_context_data(self, **kwargs):
        context = super(ExtRemoveConsumedPRSView, self).get_context_data(
            **kwargs)
        context["ext_policy_target_id"] = self.kwargs['ext_policy_target_id']
        return context

    def get_initial(self):
        return self.kwargs


class AddConsumedPRSView(forms.ModalFormView):
    form_class = policy_target_forms.AddConsumedPRSForm
    template_name = "project/policytargets/add_consumed_prs.html"

    def get_context_data(self, **kwargs):
        context = super(AddConsumedPRSView, self).get_context_data(**kwargs)
        context["policy_target_id"] = self.kwargs['policy_target_id']
        return context

    def get_initial(self):
        return self.kwargs


class RemoveConsumedPRSView(forms.ModalFormView):
    form_class = policy_target_forms.RemoveConsumedPRSForm
    template_name = "project/policytargets/remove_consumed_prs.html"

    def get_context_data(self, **kwargs):
        context = super(RemoveConsumedPRSView, self).get_context_data(**kwargs)
        context["policy_target_id"] = self.kwargs['policy_target_id']
        return context

    def get_initial(self):
        return self.kwargs


def check_ip_availability(request):
    fixed_ip = request.GET.get('fixed_ip')
    response = {'error': 'IP address is not within the allocated pool range'}
    subnets = request.GET.get('subnets')
    subnets = subnets.split(":")
    for subnet in subnets:
        subnet_details = subnet.split(",")
        try:
            if IPAddress(fixed_ip) in IPNetwork(subnet_details[0]):
                if IPAddress(fixed_ip) >= IPAddress(subnet_details[1]) and \
                        IPAddress(fixed_ip) <= IPAddress(subnet_details[2]):
                    fixed_ips = "ip_address=" + fixed_ip
                    ports = api.neutron.port_list(request,
                        tenant_id=request.user.tenant_id, fixed_ips=fixed_ips)
                    if ports:
                        response = {"inuse": False,
                                    "error": "IP address already in use"}
                    else:
                        response = {"inuse": True}
                    break
        except Exception:
            response = {'error': 'Unable to check IP availability'}
    json_string = json.dumps(response, ensure_ascii=False)
    return HttpResponse(json_string, content_type='text/json')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.core import urlresolvers
from django.forms import fields
from django.forms import TextInput
from django.forms import widgets
from django.utils.safestring import mark_safe


class DynamicMultiSelectWidget(widgets.SelectMultiple):

    """A subclass of the ``Select`` widget which renders extra attributes for
    use in callbacks to handle dynamic changes to the available choices.
    """
    _data_add_url_attr = "data-add-item-url"

    def render(self, *args, **kwargs):
        add_item_url = self.get_add_item_url()
        if add_item_url is not None:
            self.attrs[self._data_add_url_attr] = add_item_url
        return super(DynamicMultiSelectWidget, self).render(*args, **kwargs)

    def get_add_item_url(self):
        if callable(self.add_item_link):
            return self.add_item_link()
        try:
            if self.add_item_link_args:
                return urlresolvers.reverse(self.add_item_link,
                                            args=self.add_item_link_args)
            else:
                return urlresolvers.reverse(self.add_item_link)
        except urlresolvers.NoReverseMatch:
            return self.add_item_link


class DynamicMultiChoiceField(fields.MultipleChoiceField):

    """A subclass of ``ChoiceField`` with additional properties that make
    dynamically updating its elements easier.

    Notably, the field declaration takes an extra argument, ``add_item_link``
    which may be a string or callable defining the URL that should be used
    for the "add" link associated with the field.
    """
    widget = DynamicMultiSelectWidget

    def __init__(self,
                 add_item_link=None,
                 add_item_link_args=None,
                 *args,
                 **kwargs):
        super(DynamicMultiChoiceField, self).__init__(*args, **kwargs)
        self.widget.add_item_link = add_item_link
        self.widget.add_item_link_args = add_item_link_args


class CustomMultiChoiceField(DynamicMultiChoiceField):
    def validate(self, *args, **kwargs):
        return True


class DropdownEditWidget(TextInput):
    def __init__(self, data_list, name, *args, **kwargs):
        super(DropdownEditWidget, self).__init__(*args, **kwargs)
        self._name = name
        self._list = data_list
        self.attrs.update({'list': 'list__%s' % self._name})

    def render(self, name, value, attrs=None):
        text_html = super(DropdownEditWidget, self).render(
            name, value, attrs=attrs)
        data_list = '<datalist id="list__%s">' % self._name
        for item in self._list:
            data_list += '<option value="%s">' % item
        data_list += '</datalist>'
        return mark_safe(text_html + data_list)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging

from django.core.urlresolvers import reverse
from django import http
from django import shortcuts
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tables

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.instances import tables as itables
from openstack_dashboard.dashboards.project.instances import tabs

from gbpui import client

LOG = logging.getLogger(__name__)


class UpdatePTGLink(tables.LinkAction):
    name = "updatepolicy_target"
    verbose_name = _("Edit")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, policy_target):
        u = "horizon:project:policytargets:updatepolicy_target"
        base_url = reverse(u, kwargs={'policy_target_id': policy_target.id})
        return base_url


class DeletePTGLink(tables.DeleteAction):
    name = "deletepolicytarget"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Group")
    data_type_plural = _("Groups")


class AddPTGLink(tables.LinkAction):
    name = "addpolicy_target"
    verbose_name = _("Create Internal Group")
    url = "horizon:project:policytargets:addpolicy_target"
    classes = ("ajax-modal", "btn-addpolicy_target",)


class PTGsTable(tables.DataTable):
    name = tables.Column("name",
                    verbose_name=_("Name"),
                    link="horizon:project:policytargets:policy_targetdetails")
    description = tables.Column("description", verbose_name=_("Description"))
    provided_policy_rule_sets = tables.Column("provided_policy_rule_sets",
                                         sortable=False,
                                         verbose_name=_("Provided Rule Sets"))
    consumed_policy_rule_sets = tables.Column("consumed_policy_rule_sets",
                                         sortable=False,
                                         verbose_name=_("Consumed Rule Sets"))
    l2_policy_id = tables.Column("l2_policy_id",
                                 verbose_name=_("L2 Policy"))

    class Meta(object):
        name = "policy_targetstable"
        verbose_name = _("Internal Groups")
        table_actions = (AddPTGLink, DeletePTGLink)
        row_actions = (UpdatePTGLink, DeletePTGLink)


class UpdateExternalPTGLink(tables.LinkAction):
    name = "updateexternal_policy_target"
    verbose_name = _("Edit")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, ext_policy_target):
        u = "horizon:project:policytargets:update_ext_policy_target"
        base_url = reverse(u, kwargs={'ext_policy_target_id':
            ext_policy_target.id})
        return base_url


class AddExternalPTGLink(tables.LinkAction):
    name = "addexternal_policy_target"
    verbose_name = _("Create External Group")
    url = "horizon:project:policytargets:addexternal_policy_target"
    classes = ("ajax-modal", "btn-addexternal_policy_target",)


class DeleteExternalPTGLink(tables.DeleteAction):
    name = "deleteexternalpolicytarget"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Group")
    data_type_plural = _("Groups")


class ExternalPTGsTable(tables.DataTable):
    name = tables.Column("name",
                verbose_name=_("Name"),
                link="horizon:project:policytargets:ext_policy_targetdetails")
    description = tables.Column("description", verbose_name=_("Description"))
    provided_policy_rule_sets = tables.Column("provided_policy_rule_sets",
                                         sortable=False,
                                         verbose_name=_("Provided Rule Sets"))
    consumed_policy_rule_sets = tables.Column("consumed_policy_rule_sets",
                                         sortable=False,
                                         verbose_name=_("Consumed Rule Sets"))
    external_segments = tables.Column("external_segments",
                                 verbose_name=_("External Connectivity"))

    class Meta(object):
        name = "external_policy_targetstable"
        verbose_name = _("External Group")
        table_actions = (AddExternalPTGLink, DeleteExternalPTGLink)
        row_actions = (UpdateExternalPTGLink, DeleteExternalPTGLink,)


class LaunchVMLink(tables.LinkAction):
    name = "launch_vm"
    verbose_name = _("Create Member")
    classes = ("ajax-modal", "btn-addvm",)

    def get_link_url(self):
        return reverse("horizon:project:policytargets:addvm",
               kwargs={'policy_target_id':
                   self.table.kwargs['policy_target_id']})


class RemoveVMLink(tables.DeleteAction):
    data_type_singular = _("Member")
    data_type_plural = _("Members")

    def delete(self, request, instance_id):
        url = reverse("horizon:project:policytargets:policy_targetdetails",
                      kwargs={'policy_target_id':
                          self.table.kwargs['policy_target_id']})
        try:
            pts = []
            for port in api.neutron.port_list(request, device_id=instance_id):
                policytarget = client.pt_list(request,
                tenant_id=request.user.tenant_id, port_id=port.id)
                if policytarget:
                    pts.append(policytarget[0])
            for pt in pts:
                client.pt_delete(request, pt.id)
            api.nova.server_delete(request, instance_id)
            LOG.debug('Deleted instance %s successfully' % instance_id)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _('Failed to delete instance %s') % instance_id
            LOG.info(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class ConsoleLink(tables.LinkAction):
    name = "console"
    verbose_name = _("Console")
    url = "horizon:project:instances:detail"
    classes = ("btn-console",)
    policy_rules = (("compute", "compute_extension:consoles"),)

    def get_policy_target(self, request, datum=None):
        project_id = None
        if datum:
            project_id = getattr(datum, 'tenant_id', None)
        return {"project_id": project_id}

    def allowed(self, request, instance=None):
        # We check if ConsoleLink is allowed only if settings.CONSOLE_TYPE is
        # not set at all, or if it's set to any value other than None or False.
        # return bool(getattr(settings, 'CONSOLE_TYPE', True)) and
        # instance.status in ACTIVE_STATES and not is_deleting(instance)
        return True

    def get_link_url(self, datum):
        base_url = super(ConsoleLink, self).get_link_url(datum)
        tab_query_string = tabs.ConsoleTab(
            tabs.InstanceDetailTabs).get_query_string()
        return "?".join([base_url, tab_query_string])


class InstancesTable(tables.DataTable):
    name = tables.Column("name",
                         link="horizon:project:instances:detail",
                         verbose_name=_("Instance Name"))
    image_name = tables.Column("image_name", verbose_name=_("Image Name"))
    status = tables.Column("status", verbose_name=_("Status"))
    ip = tables.Column(
        itables.get_ips, verbose_name=_("IP Address"),
        attrs={'data-type': "ip"})

    def get_empty_message(self, *args, **kwargs):
        return "No members in this group, create one"

    class Meta(object):
        name = "instances"
        verbose_name = _("Members")
        table_actions = (LaunchVMLink, RemoveVMLink,)
        row_actions = (ConsoleLink, RemoveVMLink,)


class AddProvidedLink(tables.LinkAction):
    name = "add_policy_rule_set"
    verbose_name = _("Add Policy Rule Set")
    classes = ("ajax-modal", "btn-addvm",)

    def get_link_url(self):
        return reverse("horizon:project:policytargets:add_provided_prs",
            kwargs={'policy_target_id':
                self.table.kwargs['policy_target_id']})


class RemoveProvidedLink(tables.LinkAction):
    name = "remove_policy_rule_set"
    verbose_name = _("Remove Policy Rule Set")
    classes = ("ajax-modal", "btn-addvm",)

    def get_link_url(self):
        return reverse("horizon:project:policytargets:remove_provided_prs",
                       kwargs={'policy_target_id':
                           self.table.kwargs['policy_target_id']})


class ProvidedContractsTable(tables.DataTable):
    name = tables.Column("name",
             link="horizon:project:application_policy:policy_rule_set_details",
             verbose_name=_("Name"))
    description = tables.Column("description", verbose_name=_("Description"))
    policy_rules = tables.Column("policy_rules",
                                 sortable=False,
                                 verbose_name=_("Policy Rules"))

    class Meta(object):
        name = 'provided_policy_rule_sets'
        verbose_name = _("Provided Policy Rule Set")
        table_actions = (AddProvidedLink, RemoveProvidedLink,)


class ExtAddProvidedLink(AddProvidedLink):

    def get_link_url(self):
        return reverse("horizon:project:policytargets:ext_add_provided_prs",
                       kwargs={'ext_policy_target_id':
                           self.table.kwargs['ext_policy_target_id']})


class ExtRemoveProvidedLink(RemoveProvidedLink):

    def get_link_url(self):
        return reverse("horizon:project:policytargets:ext_remove_provided_prs",
                       kwargs={'ext_policy_target_id':
                           self.table.kwargs['ext_policy_target_id']})


class ExtProvidedContractsTable(ProvidedContractsTable):

    class Meta(ProvidedContractsTable.Meta):
        table_actions = (ExtAddProvidedLink, ExtRemoveProvidedLink,)


class AddConsumedLink(tables.LinkAction):
    name = "add_consumed"
    verbose_name = _("Add Consumed Policy Rule Set")
    classes = ("ajax-modal", "btn-addvm",)

    def get_link_url(self):
        return reverse("horizon:project:policytargets:add_consumed_prs",
                       kwargs={'policy_target_id':
                           self.table.kwargs['policy_target_id']})


class RemoveConsumedLink(tables.LinkAction):
    name = "remove_consumed"
    verbose_name = _("Remove Consumed Policy Rule Set")
    classes = ("ajax-modal", "btn-addvm",)

    def get_link_url(self):
        return reverse("horizon:project:policytargets:remove_consumed_prs",
                       kwargs={'policy_target_id':
                           self.table.kwargs['policy_target_id']})


class ConsumedContractsTable(tables.DataTable):
    name = tables.Column("name",
             link="horizon:project:application_policy:policy_rule_set_details",
             verbose_name=_("Name"))
    description = tables.Column("description",
                                verbose_name=_("Description"))
    policy_rules = tables.Column("policy_rules",
                                 sortable=False,
                                 verbose_name=_("Policy Rules"))

    class Meta(object):
        name = 'consumed_policy_rule_sets'
        verbose_name = _("Consumed Policy Rule Set")
        table_actions = (AddConsumedLink, RemoveConsumedLink,)


class ExtAddConsumedLink(AddConsumedLink):

    def get_link_url(self):
        return reverse("horizon:project:policytargets:ext_add_consumed_prs",
                       kwargs={'ext_policy_target_id':
                           self.table.kwargs['ext_policy_target_id']})


class ExtRemoveConsumedLink(RemoveConsumedLink):

    def get_link_url(self):
        return reverse("horizon:project:policytargets:ext_remove_consumed_prs",
                       kwargs={'ext_policy_target_id':
                           self.table.kwargs['ext_policy_target_id']})


class ExtConsumedContractsTable(ConsumedContractsTable):

    class Meta(ConsumedContractsTable.Meta):
        table_actions = (ExtAddConsumedLink, ExtRemoveConsumedLink,)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

PANEL = 'application_policy_panel'
PANEL_DASHBOARD = 'project'
PANEL_GROUP = 'GroupPolicyPanels'
ADD_PANEL = 'gbpui.panels.application_policy.panel.ApplicationPolicy'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import tables
from horizon import tabs

from gbpui import client

import forms as np_forms
import tables as np_tables
import tabs as np_tabs


class IndexView(tabs.TabView):
    tab_group_class = (np_tabs.L3PolicyTabs)
    template_name = 'project/network_policy/details_tabs.html'

    def post(self, request, *args, **kwargs):
        obj_ids = request.POST.getlist('object_ids')
        action = request.POST['action']
        obj_type = re.search('delete([0-9a-z]+)', action).group(1)
        if not obj_ids:
            obj_ids.append(re.search('([0-9a-z-]+)$', action).group(1))
        if obj_type == 'spolicy':
            for obj_id in obj_ids:
                try:
                    client.delete_networkservice_policy(request, obj_id)
                    messages.success(request,
                            _('Deleted service policy %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)
        if obj_type == 'l3policy':
            for obj_id in obj_ids:
                try:
                    client.l3policy_delete(request, obj_id)
                    messages.success(request,
                            _('Deleted L3 policy %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)
        if obj_type == 'externalconnectivity':
            for obj_id in obj_ids:
                try:
                    client.delete_externalconnectivity(request, obj_id)
                    messages.success(request,
                            _('Deleted External Connectivity %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)
        if obj_type == 'natpool':
            for obj_id in obj_ids:
                try:
                    client.delete_natpool(request, obj_id)
                    messages.success(request,
                            _('Deleted NAT Pool %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)
        return self.get(request, *args, **kwargs)


class AddL3policyView(forms.ModalFormView):
    form_class = np_forms.AddL3PolicyForm
    template_name = "project/network_policy/add_l3policy.html"

    def get_context_data(self, **kwargs):
        context = super(AddL3policyView, self).get_context_data(**kwargs)
        return context

    def get_initial(self):
        return self.kwargs


class L3PolicyUpdateView(forms.ModalFormView):
    form_class = np_forms.UpdateL3PolicyForm
    template_name = "project/network_policy/update_l3policy.html"

    def get_context_data(self, **kwargs):
        context = super(L3PolicyUpdateView, self).get_context_data(**kwargs)
        context['l3policy_id'] = self.kwargs['l3policy_id']
        return context

    def get_initial(self):
        return self.kwargs


class L3PolicyDetailsView(tables.MultiTableView):
    table_classes = (np_tables.L2PolicyTable,)
    template_name = 'project/network_policy/l3policy_details.html'

    def post(self, request, *args, **kwargs):
        obj_ids = request.POST.getlist('object_ids')
        action = request.POST['action']
        obj_type = re.search('delete([0-9a-z]+)', action).group(1)
        if not obj_ids:
            obj_ids.append(re.search('([0-9a-z-]+)$', action).group(1))
        if obj_type == 'l2policy':
            for obj_id in obj_ids:
                try:
                    client.l2policy_delete(request, obj_id)
                    messages.success(request,
                            _('Deleted L2 policy %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)
        return self.get(request, *args, **kwargs)

    def get_l2policy_table_data(self):
        l2_policies = []
        try:
            condition = {'l3_policy_id': self.kwargs['l3policy_id']}
            l2_policies = client.l2policy_list(self.request,
                    tenant_id=self.request.user.tenant_id, **condition)
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve l2 policy list.'))
        return l2_policies

    def get_context_data(self, **kwargs):
        context = super(L3PolicyDetailsView, self).get_context_data(**kwargs)
        p = client.l3policy_get(self.request, self.kwargs['l3policy_id'])
        context['l3policy'] = p
        return context


class AddL2policyView(forms.ModalFormView):
    form_class = np_forms.AddL2PolicyForm
    template_name = "project/network_policy/add_l2policy.html"

    def get_context_data(self, **kwargs):
        context = super(AddL2policyView, self).get_context_data(**kwargs)
        return context

    def get_initial(self):
        return self.kwargs


class L2PolicyUpdateView(forms.ModalFormView):
    form_class = np_forms.UpdateL2PolicyForm
    template_name = "project/network_policy/update_l2policy.html"

    def get_context_data(self, **kwargs):
        context = super(L2PolicyUpdateView, self).get_context_data(**kwargs)
        context['l2policy_id'] = self.kwargs['l2policy_id']
        return context

    def get_initial(self):
        return self.kwargs


class L2PolicyDetailsView(tabs.TabView):
    tab_group_class = (np_tabs.L2PolicyDetailsTabs)
    template_name = 'project/network_policy/details_tabs.html'


class CreateServicePolicyView(forms.ModalFormView):
    form_class = np_forms.CreateServicePolicyForm
    template_name = "project/network_policy/create_service_policy.html"

    def get_context_data(self, **kwargs):
        context = super(
            CreateServicePolicyView, self).get_context_data(**kwargs)
        return context


class AddExternalSegmentParamView(forms.ModalFormView):
    form_class = np_forms.CreateExternalSegmentParamForm
    template_name = "project/network_policy/create_external_segment_param.html"

    def get_object_id(self, params):
        return params.name


class AddNetworkServiceParamView(forms.ModalFormView):
    form_class = np_forms.CreateNetworkServiceParamForm
    template_name = "project/network_policy/create_network_service_param.html"

    def get_object_id(self, params):
        return params.name


class AddExternalRouteParamView(forms.ModalFormView):
    form_class = np_forms.CreateExternalRouteParamForm
    template_name = "project/network_policy/create_external_route_param.html"

    def get_object_id(self, params):
        return params.name


class UpdateNATPoolView(forms.ModalFormView):
    form_class = np_forms.UpdateNATPoolForm
    template_name = "project/network_policy/update_nat_pool.html"

    def get_context_data(self, **kwargs):
        context = super(
            UpdateNATPoolView, self).get_context_data(**kwargs)
        context['nat_pool_id'] = self.kwargs['nat_pool_id']
        return context

    def get_initial(self):
        return self.kwargs


class UpdateExternalConnectivityView(forms.ModalFormView):
    form_class = np_forms.UpdateExternalConnectivityForm
    template_name = "project/network_policy/update_external_connectivity.html"

    def get_context_data(self, **kwargs):
        context = super(
            UpdateExternalConnectivityView, self).get_context_data(**kwargs)
        context['external_connectivity_id'] = \
            self.kwargs['external_connectivity_id']
        return context

    def get_initial(self):
        return self.kwargs


class UpdateServicePolicyView(forms.ModalFormView):
    form_class = np_forms.UpdateServicePolicyForm
    template_name = "project/network_policy/update_service_policy.html"

    def get_context_data(self, **kwargs):
        context = super(
            UpdateServicePolicyView, self).get_context_data(**kwargs)
        context['service_policy_id'] = self.kwargs['service_policy_id']
        return context

    def get_initial(self):
        return self.kwargs


class ServicePolicyDetailsView(tabs.TabView):
    tab_group_class = (np_tabs.ServicePolicyDetailsTabs)
    template_name = 'project/network_policy/details_tabs.html'


class CreateExternalConnectivityView(forms.ModalFormView):
    form_class = np_forms.CreateExternalConnectivityForm
    template_name = "project/network_policy/create_external_connectivity.html"

    def get_context_data(self, **kwargs):
        context = super(
            CreateExternalConnectivityView, self).get_context_data(**kwargs)
        return context

    def get_success_url(self):
        return reverse('horizon:project:network_policy:index')

    def get_object_id(self, external_segment):
        return [external_segment.id]


class ExternalConnectivityDetailsView(tabs.TabView):
    tab_group_class = (np_tabs.ExternalConnectivityDetailsTabs)
    template_name = 'project/network_policy/details_tabs.html'


class CreateNATPoolView(forms.ModalFormView):
    form_class = np_forms.CreateNATPoolForm
    template_name = "project/network_policy/create_nat_pool.html"

    def get_context_data(self, **kwargs):
        context = super(
            CreateNATPoolView, self).get_context_data(**kwargs)
        return context


class NATPoolDetailsView(tabs.TabView):
    tab_group_class = (np_tabs.NATPoolDetailsTabs)
    template_name = 'project/network_policy/details_tabs.html'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import re

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import tabs

import forms as ns_forms
import tabs as ns_tabs

from gbpui import client


class IndexView(tabs.TabView):
    tab_group_class = (ns_tabs.ServiceChainTabs)
    template_name = 'project/network_services/details_tabs.html'

    def post(self, request, *args, **kwargs):
        obj_ids = request.POST.getlist('object_ids')
        action = request.POST['action']
        obj_type = re.search('delete([a-z]+)', action).group(1)
        if not obj_ids:
            obj_ids.append(re.search('([0-9a-z-]+)$', action).group(1))
        if obj_type == 'scnode':
            for obj_id in obj_ids:
                try:
                    client.delete_servicechain_node(request, obj_id)
                    messages.success(request, _('Deleted %s') % obj_id)
                except Exception as e:
                    exceptions.handle(request,
                                      _('Unable to delete . %s') % e)
        if obj_type == 'scinstance':
            for obj_id in obj_ids:
                try:
                    client.delete_servicechain_instance(request, obj_id)
                    messages.success(
                        request, _('Deleted  %s') % obj_id)
                except Exception as e:
                    exceptions.handle(request,
                                      _('Unable to delete . %s') % e)
        if obj_type == 'scspec':
            for obj_id in obj_ids:
                try:
                    client.delete_servicechain_spec(request, obj_id)
                    messages.success(request,
                                     _('Deleted %s') % obj_id)
                except Exception as e:
                    exceptions.handle(request,
                                      _('Unable to delete . %s') % e)
        if obj_type == 'serviceprofile':
            for obj_id in obj_ids:
                try:
                    client.delete_service_profile(request, obj_id)
                    messages.success(request,
                                     _('Deleted %s') % obj_id)
                except Exception as e:
                    exceptions.handle(request,
                                      _('Unable to delete . %s') % e)
        return self.get(request, *args, **kwargs)


class ServiceProfileDetailsView(tabs.TabView):
    tab_group_class = (ns_tabs.ServiceProfileDetailsTabGroup)
    template_name = 'project/network_services/details_tabs.html'


class CreateServiceProfileView(forms.ModalFormView):
    form_class = ns_forms.CreateServiceProfileForm
    template_name = "project/network_services/create_service_profile.html"

    def get_context_data(self, **kwargs):
        context = super(
            CreateServiceProfileView, self).get_context_data(**kwargs)
        return context


class CreateServiceChainNodeView(forms.ModalFormView):
    form_class = ns_forms.CreateServiceChainNodeForm
    template_name = "project/network_services/create_service_chain_node.html"

    def get_context_data(self, **kwargs):
        context = super(
            CreateServiceChainNodeView, self).get_context_data(**kwargs)
        return context


class UpdateServiceChainNodeView(forms.ModalFormView):
    form_class = ns_forms.UpdateServiceChainNodeForm
    template_name = "project/network_services/update_service_chain_node.html"

    def get_context_data(self, **kwargs):
        context = super(
            UpdateServiceChainNodeView, self).get_context_data(**kwargs)
        context['scnode_id'] = self.kwargs['scnode_id']
        return context

    def get_initial(self):
        return self.kwargs


class ServiceChainNodeDetailsView(tabs.TabView):
    tab_group_class = (ns_tabs.SCNodeDetailsTabGroup)
    template_name = 'project/network_services/details_tabs.html'


class CreateServiceChainSpecView(forms.ModalFormView):
    form_class = ns_forms.CreateServiceChainSpecForm
    template_name = "project/network_services/create_service_chain_spec.html"

    def get_context_data(self, **kwargs):
        context = super(
            CreateServiceChainSpecView, self).get_context_data(**kwargs)
        return context


class UpdateServiceChainSpecView(forms.ModalFormView):
    form_class = ns_forms.UpdateServiceChainSpecForm
    template_name = "project/network_services/update_service_chain_spec.html"

    def get_context_data(self, **kwargs):
        context = super(
            UpdateServiceChainSpecView, self).get_context_data(**kwargs)
        context['scspec_id'] = self.kwargs['scspec_id']
        return context

    def get_initial(self):
        return self.kwargs


class ServiceChainSpecDetailsView(tabs.TabView):
    tab_group_class = (ns_tabs.SCSpecDetailsTabGroup)
    template_name = 'project/network_services/details_tabs.html'


class CreateServiceChainInstanceView(forms.ModalFormView):
    form_class = ns_forms.CreateServiceChainInstanceForm
    template_name = "project/network_services/"\
        "create_service_chain_instance.html"

    def get_context_data(self, **kwargs):
        context = super(
            CreateServiceChainInstanceView, self).get_context_data(**kwargs)
        return context


class UpdateServiceChainInstanceView(forms.ModalFormView):
    form_class = ns_forms.UpdateServiceChainInstanceForm
    template_name = "project/network_services/"\
            "update_service_chain_instance.html"

    def get_context_data(self, **kwargs):
        context = super(
            UpdateServiceChainInstanceView, self).get_context_data(**kwargs)
        context['scinstance_id'] = self.kwargs['scinstance_id']
        return context

    def get_initial(self):
        return self.kwargs


class ServiceChainInstanceDetailsView(tabs.TabView):
    tab_group_class = (ns_tabs.SCInstanceDetailsTabGroup)
    template_name = 'project/network_services/details_tabs.html'
